'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var MATH_ROUND = Math.round;
var MATH_MAX = Math.max;
var MATH_MIN = Math.min;
var OBJECT_FREEZE = Object.freeze;

var DEFAULT_HEX_CODE_VALUE = '000000';
var L_THRESHOLD = Math.sqrt(1.05 * 0.05) - 0.05;
var GAMMA_THRESHOLD = 155;

/**
 * convenience function to round fraction to two digits
 *
 * @param {number} number
 * @returns {number}
 */
var roundToTwoDigits = function roundToTwoDigits(number) {
  return MATH_ROUND(number * 100) / 100;
};

/**
 * based on array of hsl / hsla values,
 * return built string of comma-separated hsl CSS values
 *
 * @param {Array} hsla
 * @returns {string}
 */
var getHslaString = function getHslaString(hsla) {
  return hsla.reduce(function (hslaValueString, hslaPart, index) {
    var isHue = index === 0;
    var isAlpha = index === 3;

    if (isHue) {
      return hslaValueString + hslaPart;
    }

    if (isAlpha) {
      return hslaValueString + ', ' + hslaPart;
    }

    return hslaValueString + ', ' + MATH_ROUND(hslaPart * 100) + '%';
  }, '');
};

/**
 * determine whether the foreground color for the text
 * used with the color as a background color should
 * be dark (preferrably black), based on general gamma guideliness
 *
 * @param {Array<number>} rgb
 * @returns {boolean}
 */
var shouldForegroundBeDark = function shouldForegroundBeDark(rgb) {
  var gammaValue = rgb.reduce(function (currentGammaValue, colorPart, colorPartIndex) {
    switch (colorPartIndex) {
      case 0:
        return currentGammaValue + colorPart * 0.299;

      case 1:
        return currentGammaValue + colorPart * 0.587;

      case 2:
        return currentGammaValue + colorPart * 0.114;
    }
  }, 0);

  return gammaValue >= GAMMA_THRESHOLD;
};

/**
 * determine whether the foreground color for the text
 * used with the color as a background color should
 * be dark (preferrably black), based on relative
 * luminance definitions in the spec:
 *
 * https://www.w3.org/TR/WCAG20/#relativeluminancedef
 *
 * @param {Array} rgb
 * @returns {boolean}
 */
var shouldForegroundBeDarkW3C = function shouldForegroundBeDarkW3C(rgb) {
  var L = rgb.reduce(function (currentL, color, colorIndex) {
    var updatedColor = color / 255;

    if (updatedColor <= 0.03928) {
      updatedColor /= 12.92;
    } else {
      updatedColor = Math.pow((updatedColor + 0.055) / 1.055, 2.4);
    }

    switch (colorIndex) {
      case 0:
        return currentL + 0.2126 * updatedColor;

      case 1:
        return currentL + 0.7152 * updatedColor;

      case 2:
        return currentL + 0.0722 * updatedColor;
    }
  }, 0);

  return L > L_THRESHOLD;
};

/**
 * converts string to integer hash value
 *
 * @param {string} string
 * @returns {number}
 */
var hashCode = function hashCode(string) {
  var hash = 0,
      index = string.length;

  for (; index--;) {
    hash = string.charCodeAt(index) + ((hash << 5) - hash);
  }

  return hash;
};

/**
 * convert integer value to hex code
 *
 * @param {number} integer
 * @returns {string}
 */
var integerToHex = function integerToHex(integer) {
  var hex = (integer >> 24 & 0xFF).toString(16) + (integer >> 16 & 0XFF).toString(16) + (integer >> 8 & 0xFF).toString(16) + (integer & 0xFF).toString(16);

  if (!hex) {
    return DEFAULT_HEX_CODE_VALUE;
  }

  if (hex.length < 6) {
    var hexCharArray = [];

    for (var index = 0; index < 6; index++) {
      hexCharArray.push(hex[index] || '0');
    }

    return hexCharArray.join('');
  }

  return hex.substring(0, 6);
};

/**
 * based on string passed, return hex code generated
 * from hashed value
 *
 * @param {string} string
 * @returns {string}
 */
var stringToHex = function stringToHex(string) {
  if (!string) {
    return DEFAULT_HEX_CODE_VALUE;
  }

  var hash = hashCode(string);

  return integerToHex(hash).substring(0, 6);
};

/**
 * build RGB color from hashed string value
 *
 * @param {string} hex
 * @returns {Array<number>}
 */
var stringToRgb = function stringToRgb(hex) {
  var red = parseInt(hex.substring(0, 2), 16);
  var green = parseInt(hex.substring(2, 4), 16);
  var blue = parseInt(hex.substring(4, 6), 16);

  return [red, green, blue];
};

/**
 * based on rgb array, return hsl array value
 *
 * @param {number} red
 * @param {number} green
 * @param {number} blue
 * @returns {Array}
 */
var rgbToHsl = function rgbToHsl(_ref) {
  var _ref2 = _slicedToArray(_ref, 3);

  var red = _ref2[0];
  var green = _ref2[1];
  var blue = _ref2[2];

  var fractionalRed = red / 255;
  var fractionalGreen = green / 255;
  var fractionalBlue = blue / 255;

  var max = MATH_MAX(fractionalRed, fractionalGreen, fractionalBlue);
  var min = MATH_MIN(fractionalRed, fractionalGreen, fractionalBlue);

  var luminance = (max + min) / 2;

  if (max === min) {
    return [0, 0, luminance];
  }

  var delta = max - min;
  var saturation = luminance > 0.5 ? delta / (2 - max - min) : delta / (max + min);

  var hue = void 0;

  switch (max) {
    case fractionalRed:
      hue = (fractionalGreen - fractionalBlue) / delta + (fractionalGreen < fractionalBlue ? 6 : 0);
      break;

    case fractionalGreen:
      hue = (fractionalBlue - fractionalRed) / delta + 2;
      break;

    case fractionalBlue:
      hue = (fractionalRed - fractionalGreen) / delta + 4;
      break;
  }

  hue *= 60;

  return [MATH_ROUND(MATH_MAX(0, hue)), roundToTwoDigits(saturation), roundToTwoDigits(luminance)];
};

/**
 * return object with a variety of color options for the developer
 *
 * @param {string} value
 * @returns {object}
 */
var createPrisma = function createPrisma(value) {
  var stringValue = '' + value;
  var hexString = stringToHex(stringValue);

  var rgbArray = stringToRgb(hexString);
  var rgbaArray = rgbArray.concat([1]);
  var hslArray = rgbToHsl(rgbArray);
  var hslaArray = hslArray.concat([1]);

  var hex = '#' + hexString;
  var rgb = 'rgb(' + rgbArray.join(', ') + ')';
  var rgba = 'rgba(' + rgbaArray.join(', ') + ')';
  var hsl = 'hsl(' + getHslaString(hslArray) + ')';
  var hsla = 'hsla(' + getHslaString(hslaArray) + ')';

  var shouldTextBeDark = shouldForegroundBeDark(rgbArray);
  var shouldTextBeDarkW3C = shouldForegroundBeDarkW3C(rgbArray);

  var prisma = Object.create(null);

  prisma.hex = hex;

  prisma.rgb = rgb;
  prisma.rgbArray = OBJECT_FREEZE(rgbArray);

  prisma.rgba = rgba;
  prisma.rgbaArray = OBJECT_FREEZE(rgbaArray);

  prisma.hsl = hsl;
  prisma.hslArray = OBJECT_FREEZE(hslArray);

  prisma.hsla = hsla;
  prisma.hslaArray = OBJECT_FREEZE(hslaArray);

  prisma.shouldTextBeDark = shouldTextBeDark;
  prisma.shouldTextBeDarkW3C = shouldTextBeDarkW3C;

  return OBJECT_FREEZE(prisma);
};

exports.default = createPrisma;
module.exports = exports['default'];